part of '../flutterx_live_data.dart';

class LiveResult<T> extends SelfLiveData<LiveResult<T>> {
  final MutableLiveData<ResultState> _state = MutableLiveData(initialValue: ResultState.idle);
  dynamic _value;

  ResultState get state => _state.value;

  T get result => _state.value == ResultState.success
      ? _value
      : throw StateError('result can only be accessed when state is ResultState.success');

  Object get error => _state.value == ResultState.error
      ? _value
      : throw StateError('error can only be accessed when state is ResultState.error');

  LiveResult() : super(reactive: true) {
    addSource(_state);
  }

  void load(Future<T> data) {
    setLoading();
    data.then(setResult, onError: setError);
  }

  void setLoading() {
    _value = null;
    _state.value = ResultState.loading;
  }

  void setResult(T value) {
    _value = value;
    _state.value = ResultState.success;
  }

  void setError(dynamic error) {
    _value = error;
    _state.value = ResultState.error;
  }

  void reset() {
    _value = null;
    _state.value = ResultState.idle;
  }
}

enum ResultState { idle, loading, error, success }
