part of '../flutterx_live_data.dart';

mixin StateObserver<T extends StatefulWidget> on State<T> {
  final List<ObserverWrapper> _disposables = [];
  int _key = 0;

  @override
  void initState() {
    super.initState();
    _doRegister();
  }

  @override
  void didUpdateWidget(T oldWidget) {
    super.didUpdateWidget(oldWidget);
    _doRegister();
  }

  @override
  void dispose() {
    super.dispose();
    for (final disposable in _disposables) disposable.dispose();
  }

  void registerObservers(BuildContext context);

  void observeAll(Iterable<LiveData> sources, void Function(List values) onChanged) =>
      _observeAllInternal(sources, onChanged, _observe);

  void _doRegister() => Dispatcher.postFrame.post(() {
        _key = 0;
        registerObservers(context);
        while (_key < _disposables.length) _disposables.removeAt(_key++).dispose();
      });

  void _observe<V>(LiveData<V> data, Observer<V> onChanged) {
    final key = _key++;
    if (key < _disposables.length) {
      final existing = _disposables[key];
      if (existing.updated(data, onChanged)) return;
      existing.dispose();
      _disposables[key] = data.observeForever(onChanged);
    } else {
      _disposables.add(data.observeForever(onChanged));
    }
  }
}
